<?php if (!defined('BASEPATH')) die('No direct script access allowed');

/**
 * fieldtype
 *
 * @package     auto_field
 * @category    Modules
 * @author      Rein de Vries <info@reinos.nl>
 * @link        http://reinos.nl
 * @copyright   Copyright (c) 2014 Reinos.nl Internet Media
 */

include(PATH_THIRD.'auto_field/config.php');


class Auto_field_ft extends EE_Fieldtype
{

    public $info = array(
        'name'      => AUTO_FIELD_NAME,
        'version'   => AUTO_FIELD_VERSION,
        'has_global_settings' => 'n'
    );

    public $settings = array();

    public $default_settings = array();

    public $has_array_data = TRUE;

    private $prefix;

    // ----------------------------------------------------------------------

    /**
     * Constructor
     *
     * @access public
     *
     * Calls the parent constructor
     */
    public function __construct()
    {
        parent::__construct();
//        if (version_compare(APP_VER, '2.1.4', '>')) { parent::__construct(); } else { parent::EE_Fieldtype(); }

        $this->prefix = AUTO_FIELD_MAP.'_';

        //load lang file
        ee()->lang->loadfile(AUTO_FIELD_MAP);

        //require settings
        require 'settings.php';
    }

    // ---------------------------------------------------------------------- 
    // After saving the content to the database
    // ---------------------------------------------------------------------- 
    /**
     * save (Native EE)
     *
     * @access public
    */
    public function post_save($data)
    {
        return $this->_post_save($data);
    }

     /**
     * Save after the saving of the entry
     *
     * @access public
    */
    public function _post_save($data)
    {

        //init TMPL
        if (!class_exists('EE_Template'))
        {
            require_once APPPATH.'libraries/Template.php';
        }
        $OLD_TMPL = isset(ee()->TMPL) ? ee()->TMPL : NULL;
        ee()->remove('TMPL');
        ee()->set('TMPL', new EE_Template());

        //set the entry data
        $entry = ee('Model')->get('ChannelEntry')->filter('entry_id', $this->content_id)->first();
        $entry_data = $this->addFieldNames($entry->toArray());


        $channel = ee('Model')->get('Channel')->filter('channel_id', $entry_data['channel_id'])->first();
        $channel_data = $channel->toArray();

        //merge with channel data
        $variables = array(array_merge($entry_data, $channel_data));
        $new_data = ee()->TMPL->parse_variables($this->settings['auto_field_pattern'], $variables);

        //reset TMPL
        ee()->remove('TMPL');
        ee()->set('TMPL', $OLD_TMPL);

        //update new data
        $entry->{'field_id_'.$this->settings['field_id']} = $new_data;
        $entry->save();
   }

    // ---------------------------------------------------------------------- 
    // Before saving the content to the database
    // ---------------------------------------------------------------------- 
    
    /**
     * save (Native EE)
     *
     * @access public
    */
    public function save($data)
    {
        return $this->_save($data);
    }


    // ---------------------------------------------------------------------- 

    /**
     * save
     *
     * @access public
    */
    private function _save($data = '')
    {        
        if(is_array($data))
        {
            $data = implode('|', $data);
        }
        
        return $data;
    }

    // ----------------------------------------------------------------------
    // Display the field for all types
    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
    */
    function display_field($data)
    {
        //generate the data
        return $this->_display_field($data);
    }
    
    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
    */
    private function _display_field($data, $type = 'default')
    {
        //save data
        $data = trim($data);

        ee()->cp->add_to_foot('
        <script>
            jQuery(function(){
                $("#'.$this->field_name.'").parent().parent().hide();
            });</script>
        ');

        //return data to display
        //add time() inside, other whise the save is not tirggered
        return form_input($this->field_name, time(), 'id="'.$this->field_name.'"');
    }

    // ----------------------------------------------------------------------

    /**
     * replace_tag
     *
     * @access public
    */
    public function replace_tag($data, $params = array(), $tagdata = FALSE)
    {
        $data = trim($data);
        if ($data == FALSE) return;

        return $data;
    }

    // ----------------------------------------------------------------------
    // Display the settings for all types
    // ----------------------------------------------------------------------
    
    /**
     * Display settings screen (Default EE)
     *
     * @access  public
     */
    function display_settings($data)
    {
        $settings_data = $this->_display_settings($data);

        if ($this->content_type() == 'grid')
        {
            return array('field_options_'.AUTO_FIELD_NAME => $settings_data);
        }

        return array(
            'field_options_'.AUTO_FIELD_MAP => array(
                'label' => 'field_options',
                'group' => AUTO_FIELD_MAP,
                'settings' => $settings_data
            )
        );
    }

    // --------------------------------------------------------------------

     /**
     * Display settings screen
     *
     * @access  public
     */
    private function _display_settings($data, $options = array(), $type = '')
    {
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                 $value = isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'];

                //global?
                if($options['global'])
                    continue;

                switch($options['type'])
                {
                    //multiselect
                    case 'm' :
                        break;

                    //select field
                    case 's' :
                        $return[] =  array(
                            'title' => $options['label'],
                            'desc' => isset($options['desc']) ? $options['desc'] : '',
                            'fields' => array(
                                $this->prefix.$options['name'] => array(
                                    'type' => 'select',
                                    'choices' => $options['options'],
                                    'value' => $value,
                                )
                            )
                        );

                        break;

                    //text field
                    default:
                    case 't' :
                        $return[] =  array(
                            'title' => $options['label'],
                            'desc' => isset($options['desc']) ? $options['desc'] : '',
                            'fields' => array(
                                $this->prefix.$options['name'] => array(
                                    'type' => 'text',
                                    'value' => $value,
                                )
                            )
                        );
                        break;
                }
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------
    // Save the settings for all types
    // ----------------------------------------------------------------------
    
    /**
     * save_settings (Default EE)
     *
     * @access public
    */
    function save_settings($data)
    {  
        return $this->_save_settings($data);
    } 

    // --------------------------------------------------------------------


    /**
     * save_settings
     *
     * @access public
    */
    private function _save_settings($data, $use_data = false)
    { 
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                //global?
                if($options['global'])
                    continue;

                $return[$this->prefix.$options['name']] =  $use_data ? $data[$this->prefix.$options['name']] : ee()->input->post($this->prefix.$options['name']);
            }
        }

        return $return;
    }

    // --------------------------------------------------------------------

    /**
     * Do additional processing after the field is created/modified. (EE)
     *
     * @access public
    */
    public function post_save_settings($data)
    {
        $this->_post_save_settings($data);
    }   

    // --------------------------------------------------------------------

    /**
     * Do additional processing after the field is created/modified.
     *
     * @access public
    */
    public function _post_save_settings($data)
    {
        //check if the field is converted to an int field        
        /*$fields = ee()->db->field_data('channel_data');

        foreach ($fields as $field)
        {
    
            if($field->name == 'field_id_'.$data['field_id'] && $field->type != 'decimal')
            {
                //change the field tupe
                ee()->db->query('ALTER TABLE  `exp_channel_data` CHANGE  `field_id_'.$data['field_id'].'`  `field_id_'.$data['field_id'].'` decimal(65,'.$this->settings['currency_decimal'].') NULL DEFAULT NULL');
                
                //update existing items to be null where they are zero
                ee()->db->where('field_id_'.$data['field_id'], '0,'.$this->settings['currency_decimal'].'');
                ee()->db->update('channel_data', array(
                  'field_id_'.$data['field_id'] => null  
                ));
            } 
        }*/

    } 

    // ----------------------------------------------------------------------
    
    /**
     * install
     *
     * @access public
    */
    function install()
    {
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                $return[$this->prefix.$options['name']] = $options['def_value'];
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * Update
     */
    function update()
    {
        return TRUE;
    }

    // ----------------------------------------------------------------------

    /**
     * display_global_settings
     *
     * @access public
    */
    function display_global_settings()
    {
        $data = array_merge($this->settings, $_POST);

        $form = '';
        foreach($this->fieldtype_settings as $field=>$options)
        {
            //global?
            if(!$options['global'])
                continue;

            switch($options['type'])
            {
                //multiselect
                case 'm' : 
                    $form .= form_label($options['label'], $options['label']).NBS.form_multiselect($this->prefix.$options['name'], $options['options'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                break;

                //select field
                case 's' : 
                    $form .= form_label($options['label'], $options['label']).NBS.form_dropdown($this->prefix.$options['name'], $options['options'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                break;

                //text field
                default:
                case 't' : 
                    $form .= form_label($options['label'], $options['label']).NBS.form_input($this->prefix.$options['name'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                break;
            }
        }

        return $form;
    }

    // ----------------------------------------------------------------------

    /**
     * save_global_settings
     *
     * @access public
    */
    function save_global_settings()
    {
        return array_merge($this->settings, $_POST);
    }

    // ---------------------------------------------------------------------- 

    /**
     * accepts_content_type (GRID EE 2.7)
     *
     * @access public
    */
    public function accepts_content_type($name)
    {
        return ($name == 'channel');
    }

    // ---------------------------------------------------------------------- 
    // Custom 
    // ----------------------------------------------------------------------
    
    /**
     * add Field names
     *
     * @access private
    */
    private function addFieldNames($entryData)
    {
        $fields = $this->preg_array_key_exists('/field_id_/', $entryData);

        if(!empty($fields))
        {
            foreach($fields as $field)
            {
                $id = str_replace('field_id_', '', $field);
                $cfield = ee('Model')->get('ChannelField')->filter('field_id', $id)->first();
                $entryData[$cfield->field_name] = $entryData[$field];
            }
        }

        return $entryData;
    }

    //serach array with regex
    private function preg_array_key_exists($pattern, $array)
    {
        if(!is_array($array))
        {
            return false;
        }
        $keys = array_keys($array);
        return preg_grep($pattern,$keys);
    }

    // ----------------------------------------------------------------------

}

/* End of file ft.auto_field.php */
/* Location: ./system/expressionengine/third_party/auto_field/ft.auto_field.php */
